clear;
clc;

% Load the data
load ini.txt;
load target.txt;

% Inputs
X = ini;
Y = target;

% Train the model
rng default
Mdl = fitrsvm(X,Y,'Standardize',true,'KernelFunction','polynomial',...
	'KernelScale','auto','KFold',5,'BoxConstraint',10,'Epsilon',0.01);
mse = kfoldLoss(Mdl)
Mdl = fitrsvm(X,Y,'Standardize',true,'KernelFunction','gaussian',...
	'KernelScale','auto','BoxConstraint',10,'Epsilon',0.01);

% Output parameters for SVR with RBF kernal
conv = Mdl.ConvergenceInfo.Converged
lStd = resubLoss(Mdl)

% Output fitting results
fit = predict(Mdl,X);

% Draw the error distribution diagram
error = Y - fit;
table_final = table(Y,fit,error,100*error./Y,...
     'VariableNames',{'real','estimation','error','relative_error'});
figure;
[errorNum,xaxis] = hist(error,[-0.3:0.05:0.3]); 
bar(xaxis,errorNum)
axis([-0.325 0.325 0 (ceil(max(errorNum)./100).*100)]);
xticks(-0.325:0.05:0.325);
set(gca,'XTickLabelRotation',90)
for i = 1:length(xaxis)
    text(xaxis(i),errorNum(i),num2str(errorNum(i),'%g'),...
    'HorizontalAlignment','center',...
    'VerticalAlignment','bottom',...
    'Fontname', 'Times new Roman', 'Fontsize', 10)
end
xlabel('Error');
ylabel('Instances','Position',[-0.425 (ceil(max(errorNum)./100).*100)/2]);
figure_FontSize=10;
set(get(gca,'XLabel'),'FontSize',figure_FontSize,'Vertical','top');
set(get(gca,'YLabel'),'FontSize',figure_FontSize,'Vertical','middle');
set(findobj('FontSize',10),'FontSize',figure_FontSize);
set(gcf,'Position',[100 100 260 220]);
set(gca,'Position',[.15 .19 .80 .74]);
set(gca, 'Fontname', 'Times new Roman', 'Fontsize', 10);
grid on;
[r1,~]=rsquare(Y,fit)

% Get the highest imperfection sensitivity
[a,~]=size(ini);
tt=zeros(a/10,1);
maxnum=zeros(a/10,1);
ty=zeros(a/10,1);
tfit=zeros(a/10,1);
for i = 1:a/10
   tt(i,1)=Y(10*(i-1)+1,1);
   for j = 1:10
      if Y(10*(i-1)+j,1)>tt(i,1)
         tt(i,1)=Y(10*(i-1)+j,1);
         maxnum(i)=j;
      end
   end
   ty(i)=Y(10*(i-1)+maxnum(i));
end
for i = 1:a/10
   tt(i,1)=fit(10*(i-1)+1,1);
   for j = 1:10
      if fit(10*(i-1)+j,1)>tt(i,1)
         tt(i,1)=fit(10*(i-1)+j,1);
         maxnum(i)=j;
      end
   end
   tfit(i)=fit(10*(i-1)+maxnum(i));
end

% Draw the error distribution diagram
error = ty - tfit;
table_final = table(ty,tfit,error,100*error./ty,...
     'VariableNames',{'real','estimation','error','relative_error'});
figure;
[errorNum,xaxis] = hist(error,[-0.3:0.05:0.3]); 
bar(xaxis,errorNum)
axis([-0.325 0.325 0 (ceil(max(errorNum)./100).*100)]);
xticks(-0.325:0.05:0.325);
set(gca,'XTickLabelRotation',90)
for i = 1:length(xaxis)
    text(xaxis(i),errorNum(i),num2str(errorNum(i),'%g'),...
    'HorizontalAlignment','center',...
    'VerticalAlignment','bottom',...
    'Fontname', 'Times new Roman', 'Fontsize', 10)
end
xlabel('Error');
ylabel('Instances','Position',[-0.425 (ceil(max(errorNum)./100).*100)/2]);
figure_FontSize=10;
set(get(gca,'XLabel'),'FontSize',figure_FontSize,'Vertical','top');
set(get(gca,'YLabel'),'FontSize',figure_FontSize,'Vertical','middle');
set(findobj('FontSize',10),'FontSize',figure_FontSize);
set(gcf,'Position',[100 100 260 220]);
set(gca,'Position',[.15 .19 .80 .74]);
set(gca, 'Fontname', 'Times new Roman', 'Fontsize', 10);
grid on;

% Load test values
load ini1.txt;
load target1.txt;

X1 = ini1;
Y1 = target1;

% Output fitting results
fit1 = predict(Mdl,X1);

% Draw the error distribution diagram
error1 = Y1 - fit1;
table_final = table(Y1,fit1,error1,100*error1./Y1,...
     'VariableNames',{'real','estimation','error','relative_error'});
figure;
[errorNum,xaxis] = hist(error1,[-0.3:0.05:0.3]); 
bar(xaxis,errorNum)
axis([-0.325 0.325 0 (ceil(max(errorNum)./100).*100)]);
xticks(-0.325:0.05:0.325);
set(gca,'XTickLabelRotation',90)
for i = 1:length(xaxis)
    text(xaxis(i),errorNum(i),num2str(errorNum(i),'%g'),...
    'HorizontalAlignment','center',...
    'VerticalAlignment','bottom',...
    'Fontname', 'Times new Roman', 'Fontsize', 10)
end
xlabel('Error');
ylabel('Instances','Position',[-0.425 (ceil(max(errorNum)./100).*100)/2]);
figure_FontSize=10;
set(get(gca,'XLabel'),'FontSize',figure_FontSize,'Vertical','top');
set(get(gca,'YLabel'),'FontSize',figure_FontSize,'Vertical','middle');
set(findobj('FontSize',10),'FontSize',figure_FontSize);
set(gcf,'Position',[100 100 260 220]);
set(gca,'Position',[.15 .19 .80 .74]);
set(gca, 'Fontname', 'Times new Roman', 'Fontsize', 10);
grid on;
[r2,~]=rsquare(Y1,fit1)

% Get the highest imperfection sensitivity
[a,~]=size(ini1);
tt1=zeros(a/10,1);
maxnum1=zeros(a/10,1);
ty1=zeros(a/10,1);
tfit1=zeros(a/10,1);
for i = 1:a/10
   tt1(i,1)=Y1(10*(i-1)+1,1);
   for j = 1:10
      if Y1(10*(i-1)+j,1)>tt1(i,1)
         tt1(i,1)=Y1(10*(i-1)+j,1);
         maxnum1(i)=j;
      end
   end
   ty1(i)=Y1(10*(i-1)+maxnum1(i));
end
for i = 1:a/10
   tt1(i,1)=fit1(10*(i-1)+1,1);
   for j = 1:10
      if fit1(10*(i-1)+j,1)>tt1(i,1)
         tt1(i,1)=fit1(10*(i-1)+j,1);
         maxnum1(i)=j;
      end
   end
   tfit1(i)=fit1(10*(i-1)+maxnum1(i));
end

% Draw the error distribution diagram
error = ty1 - tfit1;
table_final = table(ty1,tfit1,error,100*error./ty1,...
     'VariableNames',{'real','estimation','error','relative_error'});
figure;
[errorNum,xaxis] = hist(error,[-0.3:0.05:0.3]); 
bar(xaxis,errorNum)
axis([-0.325 0.325 0 (ceil(max(errorNum)./100).*100)]);
xticks(-0.325:0.05:0.325);
set(gca,'XTickLabelRotation',90)
for i = 1:length(xaxis)
    text(xaxis(i),errorNum(i),num2str(errorNum(i),'%g'),...
    'HorizontalAlignment','center',...
    'VerticalAlignment','bottom',...
    'Fontname', 'Times new Roman', 'Fontsize', 10)
end
xlabel('Error');
ylabel('Instances','Position',[-0.425 (ceil(max(errorNum)./100).*100)/2]);
figure_FontSize=10;
set(get(gca,'XLabel'),'FontSize',figure_FontSize,'Vertical','top');
set(get(gca,'YLabel'),'FontSize',figure_FontSize,'Vertical','middle');
set(findobj('FontSize',10),'FontSize',figure_FontSize);
set(gcf,'Position',[100 100 260 220]);
set(gca,'Position',[.15 .19 .80 .74]);
set(gca, 'Fontname', 'Times new Roman', 'Fontsize', 10);
grid on;